#include "sound.h"

SOUND::SOUND(int _nSamplesPerSec, int _nFftLen) {
	nSamplesPerSec = _nSamplesPerSec;
	nFftLen = _nFftLen;
	pdCorrectionValue = NULL;
	GetCorrectionValue();
};

SOUND::~SOUND() {
	if(pdCorrectionValue != NULL) {
		free(pdCorrectionValue);
	};
};

void SOUND::GetCorrectionValue() {
	int i;
	double dFrequency;
	double dCoefficient;

	if(pdCorrectionValue == NULL) {
		pdCorrectionValue = (double*)malloc(nFftLen * sizeof(double) / 2);
	} else {
		pdCorrectionValue = (double*)realloc(pdCorrectionValue, nFftLen * sizeof(double) / 2);
	};
	for(i = 0; i < nFftLen / 2; i++) {
		dFrequency = i * nSamplesPerSec / nFftLen;
		if(dFrequency < 16) {
			dCoefficient = -100;
		} else if(dFrequency < 20) {
			dCoefficient = -56.7 + (dFrequency - 16) * 6.2 / 4;
		} else if(dFrequency < 25) {
			dCoefficient = -50.5 + (dFrequency - 20) * 5.8 / 5;
		} else if(dFrequency < 31.5) {
			dCoefficient = -44.7 + (dFrequency - 25) * 5.3 / 6.5;
		} else if(dFrequency < 40) {
			dCoefficient = -39.4 + (dFrequency - 31.5) * 4.8 / 8.5;
		} else if(dFrequency < 50) {
			dCoefficient = -34.6 + (dFrequency - 40) * 4.4 / 10;
		} else if(dFrequency < 63) {
			dCoefficient = -30.2 + (dFrequency - 50) * 4.0 / 13;
		} else if(dFrequency < 80) {
			dCoefficient = -26.2 + (dFrequency - 63) * 3.7 / 17;
		} else if(dFrequency < 100) {
			dCoefficient = -22.5 + (dFrequency - 80) * 3.4 / 20;
		} else if(dFrequency < 125) {
			dCoefficient = -19.1 + (dFrequency - 100) * 3.0 / 25;
		} else if(dFrequency < 160) {
			dCoefficient = -16.1 + (dFrequency - 125) * 2.7 / 35;
		} else if(dFrequency < 200) {
			dCoefficient = -13.4 + (dFrequency - 160) * 2.5 / 40;
		} else if(dFrequency < 250) {
			dCoefficient = -10.9 + (dFrequency - 200) * 2.3 / 50;
		} else if(dFrequency < 315) {
			dCoefficient = -8.6 + (dFrequency - 250) * 2.0 / 65;
		} else if(dFrequency < 400) {
			dCoefficient = -6.6 + (dFrequency - 315) * 1.8 / 85;
		} else if(dFrequency < 500) {
			dCoefficient = -4.8 + (dFrequency - 400) * 1.6 / 100;
		} else if(dFrequency < 630) {
			dCoefficient = -3.2 + (dFrequency - 500) * 1.3 / 130;
		} else if(dFrequency < 800) {
			dCoefficient = -1.9 + (dFrequency - 630) * 1.1 / 170;
		} else if(dFrequency < 1000) {
			dCoefficient = -0.8 + (dFrequency - 800) * 0.8 / 200;
		} else if(dFrequency < 1250) {
			dCoefficient = 0 + (dFrequency - 1000) * 0.6 / 250;
		} else if(dFrequency < 1600) {
			dCoefficient = 0.6 + (dFrequency - 1250) * 0.4 / 350;
		} else if(dFrequency < 2000) {
			dCoefficient = 1.0 + (dFrequency - 1600) * 0.2 / 400;
		} else if(dFrequency < 2500) {
			dCoefficient = 1.2 + (dFrequency - 2000) * 0.1 / 500;
		} else if(dFrequency < 3150) {
			dCoefficient = 1.3 - (dFrequency - 2500) * 0.1 / 650;
		} else if(dFrequency < 4000) {
			dCoefficient = 1.2 - (dFrequency - 3150) * 0.2 / 850;
		} else if(dFrequency < 5000) {
			dCoefficient = 1.0 - (dFrequency - 4000) * 0.5 / 1000;
		} else if(dFrequency < 6300) {
			dCoefficient = 0.5 - (dFrequency - 5000) * 0.6 / 1300;
		} else if(dFrequency < 8000) {
			dCoefficient = -0.1 - (dFrequency - 6300) * 1.0 / 1700;
		} else if(dFrequency < 10000) {
			dCoefficient = -1.1 - (dFrequency - 8000) * 1.4 / 2000;
		} else if(dFrequency < 12500) {
			dCoefficient = -2.5 - (dFrequency - 10000) * 1.8 / 2500;
		} else if(dFrequency < 16000) {
			dCoefficient = -4.3 - (dFrequency - 12500) * 2.3 / 3500;
		} else if(dFrequency < 20000) {
			dCoefficient = -6.6 - (dFrequency - 16000) * 2.7 / 4000;
		} else {
			dCoefficient = -100;
		};
		pdCorrectionValue[i] = pow(10, dCoefficient / 10);
	};
};

double SOUND::GetPressure(COMPLEX_NUMBER* pcnA) {
	int i;
	double dPressure = 0;

	for(i = 0; i < (nFftLen / 2); i++) {
		dPressure += (pow(pcnA[i].r, 2) + pow(pcnA[i].i, 2)) * pdCorrectionValue[i];
	};

	return sqrt(dPressure) / nFftLen;
};

short SOUND::GetStrainedValue(double dValue) {
/*	if(dValue > 16384) {
		return (short)(sin(((dValue - 16384) / ((double)lSampleMaxValue - 16384)) * MATH_PI / 2) * 16383) + 16384;
	} else if(dValue < -16384) {
		return (short)(sin(((dValue + 16384) / ((double)lSampleMaxValue - 16384)) * MATH_PI / 2) * 16383) - 16384;
	} else {
		return (short)dValue;
	};*/
	if(dValue > 32767) {
		return 32767;
	} else if(dValue < -32768) {
		return -32768;
	} else {
		return (short)dValue;
	};
};

void SOUND::SetStrainedCurve(long _lSampleMaxValue) {
	lSampleMaxValue = _lSampleMaxValue;
};
