#include <windows.h>
#include "wave.h"
#include <stdio.h>

bool WAVE::CmpStr(const unsigned char* pucStr1, const unsigned char* pucStr2, int nSize) {
	int i;

	for(i = 0; i < nSize; i++) {
		if(pucStr1[i] != pucStr2[i]) {
			return FALSE;
		};
	};
	return TRUE;
};

bool WAVE::OpenWave(const char* pcFilePath) {

	// ϐ̐錾
	HANDLE hFile;
	DWORD dwFileSize;
	DWORD dw;
	
	// t@CJ
	hFile = CreateFile((LPCSTR)pcFilePath, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
	if(hFile == INVALID_HANDLE_VALUE) {
		return FALSE;
	};
	dwFileSize = GetFileSize(hFile, NULL);
	dwWavDataSize = dwFileSize;
	pucWavData = (unsigned char*)malloc(dwWavDataSize);
	ReadFile(hFile, pucWavData, dwWavDataSize, &dw, NULL);
	CloseHandle(hFile);

	// WAVEt@C̉
	unsigned long ulCurPos;
	unsigned long ulChunkSize;

	if(!CmpStr(pucWavData, (unsigned char*)"RIFF", 4) || !CmpStr(pucWavData + 8, (unsigned char*)"WAVE", 4)) {
		return FALSE;
	};

	ulCurPos = 0x0C;
	while(ulCurPos < dwFileSize) {
		if(CmpStr(&pucWavData[ulCurPos], (unsigned char*)"fmt ", 4)) {
			memcpy(&wfcWavFmt, &pucWavData[ulCurPos], sizeof(WAVE_FORMAT_CHUNK));
			wfcWavFmt.ulChunkSize = sizeof(WAVE_FORMAT_CHUNK) - 8;
		} else if(CmpStr(&pucWavData[ulCurPos], (unsigned char*)"data", 4)) {
			memcpy(&ulChunkSize, &pucWavData[ulCurPos + 4], 4);
			pucSoundData = &pucWavData[ulCurPos + 8];
			ulSoundDataLen = ulChunkSize;
		};
		memcpy(&ulChunkSize, &pucWavData[ulCurPos + 4], 4);
		ulCurPos += ulChunkSize + 8;
	};

	// ߂l
	return TRUE;
};

bool WAVE::SaveWave(const char* pcFilePath) {
	// ϐ̐錾
	HANDLE hFile;
	DWORD dw;
	unsigned long ul;
	
	// t@CJ
	hFile = CreateFile((LPCSTR)pcFilePath, GENERIC_WRITE, 0, NULL, CREATE_NEW, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
	if(hFile == INVALID_HANDLE_VALUE) {
		hFile = CreateFile((LPCSTR)pcFilePath, GENERIC_WRITE, 0, NULL, TRUNCATE_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
		if(hFile == INVALID_HANDLE_VALUE) {
			return FALSE;
		};
	};
	WriteFile(hFile, "RIFF", 4, &dw, NULL);
	ul = sizeof(WAVE_FORMAT_CHUNK) + ulSoundDataLen + 12;
	WriteFile(hFile, &ul, 4, &dw, NULL);
	WriteFile(hFile, "WAVE", 4, &dw, NULL);
	WriteFile(hFile, &wfcWavFmt, sizeof(WAVE_FORMAT_CHUNK), &dw, NULL);
	WriteFile(hFile, "data", 4, &dw, NULL);
	WriteFile(hFile, &ulSoundDataLen, 4, &dw, NULL);
	WriteFile(hFile, pucSoundData, ulSoundDataLen, &dw, NULL);
	CloseHandle(hFile);

	return TRUE;
};

short WAVE::GetSampleValue(unsigned long ulSampleNum, int nChannel) {
	short s;

	if(wfcWavFmt.usBitsPerSample == 16) {
		memcpy(&s, &pucSoundData[wfcWavFmt.usBlockAlign * ulSampleNum + wfcWavFmt.usBitsPerSample * nChannel / 8], 2);
	} else {
		s = (short)pucSoundData[wfcWavFmt.usBlockAlign * ulSampleNum + wfcWavFmt.usBitsPerSample * nChannel / 8] - 128;
	};

	return s;
};

void WAVE::SetSampleValue(short sValue, unsigned long ulSampleNum, int nChannel) {
	if(wfcWavFmt.usBitsPerSample == 16) {
		memcpy(&pucSoundData[wfcWavFmt.usBlockAlign * ulSampleNum + wfcWavFmt.usBitsPerSample * nChannel / 8], &sValue, 2);
	} else {
		pucSoundData[wfcWavFmt.usBlockAlign * ulSampleNum + wfcWavFmt.usBitsPerSample * nChannel / 8] = (unsigned char)(sValue + 128);
	};
};

void WAVE::CutSilence() {
	unsigned long i = 0;
	unsigned long nStart;
	unsigned long nEnd;
	unsigned long ulNumOfSamples = ulSoundDataLen / wfcWavFmt.usBlockAlign;
	short sLeft;
	short sRight;

	if(wfcWavFmt.usChannels == 2) {
		do {
			sLeft = GetSampleValue(i, 0);
			sRight = GetSampleValue(i, 1);
			i++;
		} while((sLeft > -256) && (sLeft < 256) && (sRight > -256) && (sRight < 256) && (i < ulNumOfSamples));
	} else {
		do {
			sLeft = GetSampleValue(i, 0);
			sRight = GetSampleValue(i, 1);
			i++;
		} while((sLeft > -256) && (sLeft < 256) && (i < ulNumOfSamples));
	};
	if(i < wfcWavFmt.ulSamplesPerSec * 0.5) {
		nStart = 0;
	} else {
		nStart = i - wfcWavFmt.ulSamplesPerSec * 0.5;
	};

	nEnd = ulNumOfSamples - 1;
	if(wfcWavFmt.usChannels == 2) {
		while(i < ulNumOfSamples) {
			sLeft = GetSampleValue(i, 0);
			sRight = GetSampleValue(i, 1);
			if((sLeft < -256) || (sLeft > 256) || (sRight < -256) || (sRight > 256)) {
				nEnd = i;
			};
			i++;
		};
	} else {
		while(i < ulNumOfSamples) {
			sLeft = GetSampleValue(i, 0);
			if((sLeft < -256) || (sLeft > 256)) {
				nEnd = i;
			};
			i++;
		};
	};
	if(i > ulNumOfSamples - wfcWavFmt.ulSamplesPerSec * 0.5 - 1) {
		nEnd = ulNumOfSamples - 1;
	} else {
		nEnd = i + wfcWavFmt.ulSamplesPerSec * 0.5;
	};

	pucSoundData = &pucSoundData[nStart * wfcWavFmt.usBlockAlign];
	dwWavDataSize = nEnd * wfcWavFmt.usBlockAlign;
	ulSoundDataLen = (nEnd - nStart) * wfcWavFmt.usBlockAlign;
};

void WAVE::Karaokize() {
	short sValue;
	unsigned long ulNumOfSamples = ulSoundDataLen / wfcWavFmt.usBlockAlign;
	unsigned long i = 0;

	for(i = 0; i < ulNumOfSamples; i++) {
		sValue = (GetSampleValue(i, 0) - GetSampleValue(i, 1)) / 2;
		SetSampleValue(sValue, i, 0);
		SetSampleValue(sValue, i, 1);
	};
};

void WAVE::ConvertFormat(unsigned short _usChannels, unsigned long _ulSamplesPerSec, unsigned short _usBitsPerSample) {

	// ϊ邩ǂ̌vZ
	if((wfcWavFmt.usChannels == _usChannels) && (wfcWavFmt.ulSamplesPerSec == _ulSamplesPerSec) && (wfcWavFmt.usBitsPerSample == _usBitsPerSample)) {
		return;
	};

	// ϊ̃TCY̌vZEq[v̈̍Ċm
	unsigned long ulNewSoundDataLen = ulSoundDataLen;
	DWORD dwNewWavDataSize;
	short sValue;

	if((_usChannels == 2) && (wfcWavFmt.usChannels == 1)) {
		ulNewSoundDataLen <<= 1;
	} else if((_usChannels == 1) && (wfcWavFmt.usChannels == 2)) {
		ulNewSoundDataLen >>= 1;
	} else if(_usChannels > 2) {
		return;
	};

	if((_usBitsPerSample == 16) && (wfcWavFmt.usBitsPerSample == 8)) {
		ulNewSoundDataLen <<= 1;
	} else if((_usBitsPerSample == 8) && (wfcWavFmt.usBitsPerSample == 16)) {
		ulNewSoundDataLen >>= 1;
	} else if((_usBitsPerSample != 16) && (_usBitsPerSample != 8)) {
		return;
	};

	if(wfcWavFmt.ulSamplesPerSec != _ulSamplesPerSec) {
		ulNewSoundDataLen = (unsigned long)(((double)_ulSamplesPerSec / (double)wfcWavFmt.ulSamplesPerSec) * (double)ulNewSoundDataLen);
	};

	dwNewWavDataSize = dwWavDataSize - ulSoundDataLen + ulNewSoundDataLen;
	if(dwNewWavDataSize > dwWavDataSize) {
		pucWavData = (unsigned char*)realloc(pucWavData, dwNewWavDataSize);
		pucSoundData = &pucWavData[dwWavDataSize - ulSoundDataLen];
	};

	// `l̕ϊ
	unsigned long i;
	short j;
	unsigned long ulNumOfSamples = ulSoundDataLen / wfcWavFmt.usBlockAlign;

	if((_usChannels == 2) && (wfcWavFmt.usChannels == 1)) {
		for(i = ulNumOfSamples - 1; (long)i != -1; i--) {
			sValue = GetSampleValue(i, 0);
			SetSampleValue(sValue, i << 1, 0);
			SetSampleValue(sValue, i << 1, 1);
		};
		wfcWavFmt.usChannels = 2;
		wfcWavFmt.usBlockAlign <<= 1;
		wfcWavFmt.ulAvgBytesPerSec <<= 1;
	} else if((_usChannels == 1) && (wfcWavFmt.usChannels == 2)) {
		for(i = 0; i < ulNumOfSamples; i += 2) {
			sValue = GetSampleValue(i, 0);
			SetSampleValue(sValue, i >> 1, i % 2);
		};
		wfcWavFmt.usChannels = 1;
		wfcWavFmt.usBlockAlign >>= 1;
		wfcWavFmt.ulAvgBytesPerSec >>= 1;
	};

	// rbg̕ϊ
	if((_usBitsPerSample == 16) && (wfcWavFmt.usBitsPerSample == 8)) {
		for(i = ulNumOfSamples - 1; (long)i != -1; i--) {
			for(j = 0; j < wfcWavFmt.usChannels; j++) {
				sValue = GetSampleValue(i, j);
				sValue <<=  8;
				memcpy(&pucSoundData[wfcWavFmt.usChannels * i * 2 + 2 * j], &sValue, 2);
			};
		};
		wfcWavFmt.usBitsPerSample = 16;
		wfcWavFmt.usBlockAlign <<= 1;
		wfcWavFmt.ulAvgBytesPerSec <<= 1;
	} else if((_usBitsPerSample == 8) && (wfcWavFmt.usBitsPerSample == 16)) {
		for(i = 0; i < ulNumOfSamples; i++) {
			for(j = 0; j < wfcWavFmt.usChannels; j++) {
				sValue = GetSampleValue(i, j);
				sValue >>=  8;
				SetSampleValue(sValue, i << 1, 0);
				pucSoundData[wfcWavFmt.usChannels * i + j] = (unsigned char)(sValue + 128);
			};
		};
		wfcWavFmt.usBitsPerSample = 8;
		wfcWavFmt.usBlockAlign >>= 1;
		wfcWavFmt.ulAvgBytesPerSec >>= 1;
	};

	// rbg[g̕ϊ
	double dRateRatio = (double)_ulSamplesPerSec / (double)wfcWavFmt.ulSamplesPerSec;
	unsigned long ulNewNumOfSamples = (unsigned long)(dRateRatio * (double)ulNumOfSamples);

	if(ulNumOfSamples < ulNewNumOfSamples) {
		for(i = ulNewNumOfSamples - 1; (long)i != -1; i--) {
			for(j = 0; j < wfcWavFmt.usChannels; j++) {
				sValue = GetSampleValue((unsigned long)((double)i / dRateRatio), j);
				SetSampleValue(sValue, i, j);
			};
		};
		wfcWavFmt.ulSamplesPerSec = _ulSamplesPerSec;
		wfcWavFmt.ulAvgBytesPerSec = (unsigned long)((double)wfcWavFmt.ulAvgBytesPerSec * dRateRatio);
	} else if(ulNumOfSamples > ulNewNumOfSamples) {
		for(i = 0; i < ulNewNumOfSamples; i++) {
			for(j = 0; j < wfcWavFmt.usChannels; j++) {
				sValue = GetSampleValue((unsigned long)((double)i / dRateRatio), j);
				SetSampleValue(sValue, i, j);
			};
		};
		wfcWavFmt.ulSamplesPerSec = _ulSamplesPerSec;
		wfcWavFmt.ulAvgBytesPerSec = (unsigned long)((double)wfcWavFmt.ulAvgBytesPerSec * dRateRatio);
	};

	if(dwNewWavDataSize < dwWavDataSize) {
		pucWavData = (unsigned char*)realloc(pucWavData, dwNewWavDataSize);
		pucSoundData = &pucWavData[dwWavDataSize - ulSoundDataLen];
	};
	ulSoundDataLen = ulNewSoundDataLen;
};

WAVE::WAVE() {
	pucWavData = NULL;
};

WAVE::~WAVE() {
	if(pucWavData != NULL) {
		free(pucWavData);
	};
};
